@@ -42,7 +42,6 @@
 #include <math.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <pango/pangoft2.h>
 
 #define LOAD_BUFFER_SIZE 65536
 
@@ -296,7 +295,7 @@ eel_gdk_pixbuf_average_value (GdkPixbuf *pixbuf)
 	int row_stride;
 	const guchar *pixels, *p;
 	int r, g, b, a;
-	guint dividend;
+	guint64 dividend;
 	guint width, height;
 
 	width = gdk_pixbuf_get_width (pixbuf);
@@ -404,6 +403,37 @@ eel_gdk_pixbuf_scale_down_to_fit (GdkPixbuf *pixbuf, int max_width, int max_heig
 	}
 }
 
+double
+eel_gdk_scale_to_min_factor (int width, int height,
+			     int min_width, int min_height,
+			     int *scaled_width, int *scaled_height)
+{
+	double scale_factor;
+
+	scale_factor = MAX (min_width / (double) width, min_height / (double) height);
+
+	*scaled_width  = floor (width * scale_factor + .5);
+	*scaled_height = floor (height * scale_factor + .5);
+
+	return scale_factor;
+}
+
+/* Returns a scaled copy of pixbuf, preserving aspect ratio. The copy will
+ * be scaled as small as possible without going under the specified width and height.
+ */
+GdkPixbuf *
+eel_gdk_pixbuf_scale_to_min (GdkPixbuf *pixbuf, int min_width, int min_height)
+{
+	int scaled_width;
+	int scaled_height;
+
+	eel_gdk_scale_to_min_factor (gdk_pixbuf_get_width(pixbuf), gdk_pixbuf_get_height(pixbuf),
+				     min_width, min_height,
+				     &scaled_width, &scaled_height);
+
+	return gdk_pixbuf_scale_simple (pixbuf, scaled_width, scaled_height, GDK_INTERP_BILINEAR);	
+}
+
 /**
  * eel_gdk_pixbuf_is_valid:
  * @pixbuf: A GdkPixbuf
@@ -593,34 +623,16 @@ eel_gdk_pixbuf_draw_to_drawable (const GdkPixbuf *pixbuf,
 	target.x1 = target.x0 + MIN (target_width, source_width);
 	target.y1 = target.y0 + MIN (target_height, source_height);
 
-	if (gdk_pixbuf_get_has_alpha (pixbuf)) {
-		gdk_pixbuf_render_to_drawable_alpha ((GdkPixbuf *) pixbuf,
-						     drawable,
-						     source.x0,
-						     source.y0,
-						     target.x0,
-						     target.y0,
-						     target.x1 - target.x0,
-						     target.y1 - target.y0,
-						     alpha_compositing_mode,
-						     alpha_threshold,
-						     dither,
-						     0,
-						     0);
-	} else {
-		gdk_pixbuf_render_to_drawable ((GdkPixbuf *) pixbuf,
-					       drawable,
-					       gc,
-					       source.x0,
-					       source.y0,
-					       target.x0,
-					       target.y0,
-					       target.x1 - target.x0,
-					       target.y1 - target.y0,
-					       dither,
-					       0,
-					       0);
-	}
+	gdk_draw_pixbuf (drawable, gc, (GdkPixbuf *) pixbuf,
+			 source.x0,
+			 source.y0,
+			 target.x0,
+			 target.y0,
+			 target.x1 - target.x0,
+			 target.y1 - target.y0,
+			 dither,
+			 0,
+			 0);
 }
 
 /**
@@ -1256,137 +1268,6 @@ eel_gdk_pixbuf_intersect (const GdkPixbuf *pixbuf,
 	return intersection;
 }
 
-/**
- * eel_gdk_pixbuf_draw_layout_clipped:
- * @pixbuf: the pixbuf to render to
- * @region: text crop region
- * @color: the color to render the text
- * @layout: the text
- * 
- * This method renders the text in @layout into the
- * @pixbuf with @color cropping it to @region.
- **/
-void
-eel_gdk_pixbuf_draw_layout_clipped (GdkPixbuf   *pixbuf,
-				    ArtIRect     region,
-				    guint32      color,
-				    PangoLayout *layout)
-{
-	PangoRectangle ink_rect;
-	FT_Bitmap ink_bitmap;
-	GdkPixbuf *ink_pixbuf;
-	int bx, by;
-	int pixbuf_width, pixbuf_height;
-	const guint8 *ps;
-	guint8 *pd;
-	guint8 r, g, b;
-
-	if (region.x1 <= region.x0 || region.y1 <= region.y0) {
-		return;
-	}
-	
-	pango_layout_get_pixel_extents (layout, &ink_rect, NULL);
-
-	ink_rect.x += region.x0;
-	ink_rect.y += region.y0;
-
-	/* Do an intersect operation the long way.
-	 * Sad to write this all out like this.
-	 */
-
-	if (ink_rect.x < 0) {
-		ink_rect.width += ink_rect.x;
-		ink_rect.x = 0;
-	}
-
-	if (ink_rect.y < 0) {
-		ink_rect.height += ink_rect.y;
-		ink_rect.y = 0;
-	}
-
-	pixbuf_width = gdk_pixbuf_get_width (pixbuf);
-	if (ink_rect.x + ink_rect.width > pixbuf_width) {
-		ink_rect.width = pixbuf_width - ink_rect.x;
-		if (ink_rect.width <= 0) {
-			return;
-		}
-	}
-
-	pixbuf_height = gdk_pixbuf_get_height (pixbuf);
-	if (ink_rect.y + ink_rect.height > pixbuf_height) {
-		ink_rect.height = pixbuf_height - ink_rect.y;
-		if (ink_rect.height <= 0) {
-			return;
-		}
-	}
-
-	ink_bitmap.rows = ink_rect.height;
-	ink_bitmap.width = ink_rect.width;
-	ink_bitmap.pitch = (ink_bitmap.width + 3) & ~3;
-	ink_bitmap.buffer = g_malloc0 (ink_bitmap.rows * ink_bitmap.pitch);
-	ink_bitmap.num_grays = 0x100;
-	ink_bitmap.pixel_mode = ft_pixel_mode_grays;
-
-	pango_ft2_render_layout (&ink_bitmap, layout,
-				 region.x0 - ink_rect.x,
-				 region.y0 - ink_rect.y);
-
-	ink_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
-				     ink_bitmap.width, ink_bitmap.rows);
-
-	r = EEL_RGBA_COLOR_GET_R (color);
-	g = EEL_RGBA_COLOR_GET_G (color);
-	b = EEL_RGBA_COLOR_GET_B (color);
-	
-	for (by = 0; by < ink_bitmap.rows; by++) {
-	        pd = gdk_pixbuf_get_pixels (ink_pixbuf)
-			+ by * gdk_pixbuf_get_rowstride (ink_pixbuf);
-		ps = ink_bitmap.buffer + by * ink_bitmap.pitch;
-		for (bx = 0; bx < ink_bitmap.width; bx++) {
-			*pd++ = r;
-		        *pd++ = g;
-			*pd++ = b;
-			*pd++ = *ps++;
-		}
-	}
-
-	g_free (ink_bitmap.buffer);
-
-	gdk_pixbuf_composite (ink_pixbuf, pixbuf,
-			      ink_rect.x, ink_rect.y,
-			      MIN (region.x1 - region.x0, ink_bitmap.width),
-			      MIN (region.y1 - region.y0, ink_bitmap.rows),
-			      ink_rect.x, ink_rect.y,
-			      1.0, 1.0, GDK_INTERP_BILINEAR, 0xFF);
-
-	g_object_unref (ink_pixbuf);
-}
-
-/**
- * eel_gdk_pixbuf_draw_layout
- * @pixbuf:    the pixbuf on which to draw the layout
- * @x:         the X position of the left of the layout (in pixels)
- * @y:         the Y position of the top of the layout (in pixels)
- * @color:     the color to use to draw the text, in "eel RGB" format
- * @layout:    a #PangoLayout
- *
- * Render a #PangoLayout onto a #GdkPixbuf.
- */
-void
-eel_gdk_pixbuf_draw_layout (GdkPixbuf *pixbuf,
-			    int x, int y,
-			    guint32 color,
-			    PangoLayout *layout)
-{
-	ArtIRect region;
-
-	region.x0 = x;
-	region.y0 = y;
-	region.x1 = region.y1 = G_MAXINT;
-
-	eel_gdk_pixbuf_draw_layout_clipped (pixbuf, region, color, layout);
-}
-
 GdkPixbuf *
 eel_gdk_pixbuf_scale_down (GdkPixbuf *pixbuf,
 			   int dest_width,
