@@ -37,6 +37,7 @@
 #include <sys/time.h>
 #include <sys/utsname.h>
 #include <time.h>
+#include <locale.h>
 
 /* Legal conversion specifiers, as specified in the C standard. */
 #define C_STANDARD_STRFTIME_CHARACTERS "aAbBcdHIjmMpSUwWxXyYZ"
@@ -53,32 +54,6 @@ typedef struct {
 
 static GList *hash_tables_to_free_at_exit;
 
-/**
- * eel_setenv:
- * 
- * Adds "*name=*value" to the environment
- *
- * Returns: see "putenv" - 
- * 
- **/
-int
-eel_setenv (const char *name, const char *value, gboolean overwrite)
-{
-	return g_setenv (name, value, overwrite);
-}
-
-/**
- * eel_unsetenv:
- * 
- * Removes *name from the environment
- * 
- **/
-void
-eel_unsetenv (const char *name)
-{
-	g_unsetenv (name);
-}
-
 /**
  * eel_g_date_new_tm:
  * 
@@ -1208,19 +1183,17 @@ eel_self_check_glib_extensions (void)
 	memset (huge_string, 'a', 10000);
 	huge_string[10000] = '\0';
 
+	setlocale (LC_TIME, "C");
+
 	EEL_CHECK_STRING_RESULT (test_strftime ("", 2000, 1, 1, 0, 0, 0), "");
 	EEL_CHECK_STRING_RESULT (test_strftime (huge_string, 2000, 1, 1, 0, 0, 0), huge_string);
 	EEL_CHECK_STRING_RESULT (test_strftime ("%%", 2000, 1, 1, 1, 0, 0), "%");
 	EEL_CHECK_STRING_RESULT (test_strftime ("%%%%", 2000, 1, 1, 1, 0, 0), "%%");
-	/* localizers: These strings are part of the strftime
-	 * self-check code and must be changed to match what strtfime
-	 * yields. The first one is "%m/%d/%y, %I:%M %p".
-	 */
-	EEL_CHECK_STRING_RESULT (test_strftime ("%m/%d/%y, %I:%M %p", 2000, 1, 1, 1, 0, 0), _("01/01/00, 01:00 AM"));
-	/* The second one is "%-m/%-d/%-y, %-I:%M %p". */
-	EEL_CHECK_STRING_RESULT (test_strftime ("%-m/%-d/%y, %-I:%M %p", 2000, 1, 1, 1, 0, 0), _("1/1/00, 1:00 AM"));
-	/* The third one is "%_m/%_d/%_y, %_I:%M %p". */
-	EEL_CHECK_STRING_RESULT (test_strftime ("%_m/%_d/%y, %_I:%M %p", 2000, 1, 1, 1, 0, 0), _(" 1/ 1/00,  1:00 AM"));
+	EEL_CHECK_STRING_RESULT (test_strftime ("%m/%d/%y, %I:%M %p", 2000, 1, 1, 1, 0, 0), "01/01/00, 01:00 AM");
+	EEL_CHECK_STRING_RESULT (test_strftime ("%-m/%-d/%y, %-I:%M %p", 2000, 1, 1, 1, 0, 0), "1/1/00, 1:00 AM");
+	EEL_CHECK_STRING_RESULT (test_strftime ("%_m/%_d/%y, %_I:%M %p", 2000, 1, 1, 1, 0, 0), " 1/ 1/00,  1:00 AM");
+
+	setlocale (LC_TIME, "");
 
 	g_free (huge_string);
 
