@@ -533,6 +533,7 @@ get_all_applications_from_dir (GMenuTreeDirectory *directory,
 {
 	GSList *contents;
 	GSList *l;
+	const char *exec;
 
 	contents = gmenu_tree_directory_get_contents (directory);
 
@@ -546,7 +547,15 @@ get_all_applications_from_dir (GMenuTreeDirectory *directory,
 				break;
 
 			case GMENU_TREE_ITEM_ENTRY:
-				list = g_slist_prepend (list, item);
+				exec = gmenu_tree_entry_get_exec ((GMenuTreeEntry*)item);
+				/* Only add items that actually accept a file */
+				if (exec != NULL &&
+				    (strstr (exec,"%U") != NULL ||
+				     strstr (exec,"%u") != NULL ||
+				     strstr (exec,"%f") != NULL ||
+				     strstr (exec,"%F") != NULL)) {
+					list = g_slist_prepend (list, item);
+				}
 				break;
 
 			default:
@@ -582,6 +591,77 @@ get_all_applications (void)
 	return retval;
 }
 
+static gboolean
+eel_open_with_search_equal_func (GtkTreeModel *model,
+				 int column,
+				 const char *key,
+				 GtkTreeIter *iter,
+				 gpointer user_data)
+{
+	char *normalized_key;
+	char *name, *normalized_name;
+	char *path, *normalized_path;
+	char *basename, *normalized_basename;
+	gboolean ret;
+
+	if (key != NULL) {
+		normalized_key = g_utf8_casefold (key, -1);
+		g_assert (normalized_key != NULL);
+
+		ret = TRUE;
+
+		gtk_tree_model_get (model, iter,
+				    COLUMN_NAME, &name,
+				    COLUMN_PATH, &path,
+				    -1);
+
+		if (name != NULL) {
+			normalized_name = g_utf8_casefold (name, -1);
+			g_assert (normalized_name != NULL);
+
+			if (strncmp (normalized_name, normalized_key, strlen (normalized_key)) == 0) {
+				ret = FALSE;
+			}
+
+			g_free (normalized_name);
+		}
+
+		if (ret && path != NULL) {
+			/* chop any argument specifiers */
+			if (strchr (path, ' ') != NULL) {
+				*(strchr (path, ' ')) = '\0';
+			}
+
+			normalized_path = g_utf8_casefold (path, -1);
+			g_assert (normalized_path != NULL);
+
+			basename = g_path_get_basename (path);
+			g_assert (basename != NULL);
+
+			normalized_basename = g_utf8_casefold (basename, -1);
+			g_assert (normalized_basename != NULL);
+
+			if (strncmp (normalized_path, normalized_key, strlen (normalized_key)) == 0 ||
+			    strncmp (normalized_basename, normalized_key, strlen (normalized_key)) == 0) {
+				ret = FALSE;
+			}
+
+			g_free (basename);
+			g_free (normalized_basename);
+			g_free (normalized_path);
+		}
+
+		g_free (name);
+		g_free (path);
+		g_free (normalized_key);
+
+		return ret;
+	} else {
+		return TRUE;
+	}
+}
+
+
 
 static gboolean
 eel_open_with_dialog_add_items_idle (EelOpenWithDialog *dialog)
@@ -636,6 +716,9 @@ eel_open_with_dialog_add_items_idle (EelOpenWithDialog *dialog)
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW (dialog->details->program_list), 
 				 GTK_TREE_MODEL (dialog->details->program_list_store));
+	gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (dialog->details->program_list),
+					     eel_open_with_search_equal_func,
+					     NULL, NULL);
 
 	renderer = gtk_cell_renderer_pixbuf_new ();
 	column = gtk_tree_view_column_new ();
@@ -957,6 +1040,7 @@ set_uri_and_mime_type (EelOpenWithDialog *dialog,
 {
 	char *label;
 	char *name;
+	char *emname;
 	GnomeVFSURI *vfs_uri;
 
 	dialog->details->uri = g_strdup (uri);
@@ -999,7 +1083,9 @@ set_uri_and_mime_type (EelOpenWithDialog *dialog,
 		dialog->details->mime_description = description;
 	}
 
-	label = g_strdup_printf (_("Open <i>%s</i> and other files of type \"%s\" with:"), name, dialog->details->mime_description);
+	emname = g_strdup_printf ("<i>%s</i>", name);
+	label = g_strdup_printf (_("Open %s and other files of type \"%s\" with:"), emname, dialog->details->mime_description);
+	g_free (emname);
 
 	gtk_label_set_markup (GTK_LABEL (dialog->details->label), label);
 
@@ -1042,7 +1128,7 @@ eel_open_with_dialog_get_type (void)
 	static GType type = 0;
 
 	if (!type) {
-		static const GTypeInfo info = {
+		const GTypeInfo info = {
 			sizeof (EelOpenWithDialogClass),
 			NULL,
 			NULL,
