@@ -25,17 +25,19 @@
 #include <config.h>
 #include "eel-stock-dialogs.h"
 
+#include "eel-alert-dialog.h"
 #include "eel-glib-extensions.h"
 #include "eel-gnome-extensions.h"
 #include "eel-string.h"
+#include "eel-i18n.h"
 #include <gtk/gtkbox.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmessagedialog.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkstock.h>
-#include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-uidefs.h>
+#include <libgnomeui/gnome-authentication-manager.h>
 
 #define TIMED_WAIT_STANDARD_DURATION 2000
 #define TIMED_WAIT_MIN_TIME_UP 3000
@@ -49,7 +51,6 @@ typedef struct {
 	gpointer callback_data;
 
 	/* Parameters for creation of the window. */
-	char *window_title;
 	char *wait_message;
 	GtkWindow *parent_window;
 
@@ -91,28 +92,10 @@ timed_wait_hash_equal (gconstpointer value1, gconstpointer value2)
 		&& wait1->callback_data == wait2->callback_data;
 }
 
-static void
-add_label_to_dialog (GtkDialog *dialog, const char *message)
-{
-	GtkLabel *message_widget;
-
-	if (message == NULL) {
-		return;
-	}
-	
-	message_widget = GTK_LABEL (gtk_label_new (message));
-	gtk_label_set_line_wrap (message_widget, TRUE);
-	gtk_label_set_justify (message_widget,
-			       GTK_JUSTIFY_LEFT);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
-			    GTK_WIDGET (message_widget),
-			    TRUE, TRUE, GNOME_PAD);
-}
-
 static void
 timed_wait_delayed_close_destroy_dialog_callback (GtkObject *object, gpointer callback_data)
 {
-	gtk_timeout_remove (GPOINTER_TO_UINT (callback_data));
+	g_source_remove (GPOINTER_TO_UINT (callback_data));
 }
 
 static gboolean
@@ -136,19 +119,18 @@ static void
 timed_wait_free (TimedWait *wait)
 {
 	guint delayed_close_handler_id;
-	guint32 time_up;
+	guint64 time_up;
 
 	g_assert (g_hash_table_lookup (timed_wait_hash_table, wait) != NULL);
 
 	g_hash_table_remove (timed_wait_hash_table, wait);
 
-	g_free (wait->window_title);
 	g_free (wait->wait_message);
 	if (wait->parent_window != NULL) {
 		gtk_widget_unref (GTK_WIDGET (wait->parent_window));
 	}
 	if (wait->timeout_handler_id != 0) {
-		gtk_timeout_remove (wait->timeout_handler_id);
+		g_source_remove (wait->timeout_handler_id);
 	}
 	if (wait->dialog != NULL) {
 		/* Make sure to detach from the "destroy" signal, or we'll
@@ -162,7 +144,7 @@ timed_wait_free (TimedWait *wait)
 		time_up = (eel_get_system_time () - wait->dialog_creation_time) / 1000;
 		
 		if (time_up < TIMED_WAIT_MIN_TIME_UP) {
-			delayed_close_handler_id = gtk_timeout_add (TIMED_WAIT_MIN_TIME_UP - time_up,
+			delayed_close_handler_id = g_timeout_add (TIMED_WAIT_MIN_TIME_UP - time_up,
 			                                            timed_wait_delayed_close_timeout_callback,
 			                                            wait->dialog);
 			g_object_set_data (G_OBJECT (wait->dialog),
@@ -201,6 +183,14 @@ timed_wait_dialog_destroy_callback (GtkObject *object, gpointer callback_data)
 	}
 }
 
+static void
+trash_dialog_response_callback (GtkDialog *dialog,
+				int response_id,
+				TimedWait *wait)
+{
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
 static gboolean
 timed_wait_callback (gpointer callback_data)
 {
@@ -210,11 +200,21 @@ timed_wait_callback (gpointer callback_data)
 
 	wait = callback_data;
 
+	if (gnome_authentication_manager_dialog_is_visible ()) {
+		return TRUE;
+	}
+	
 	/* Put up the timed wait window. */
 	button = wait->cancel_callback != NULL ? GTK_STOCK_CANCEL : GTK_STOCK_OK;
-	dialog = GTK_DIALOG (gtk_dialog_new_with_buttons (wait->window_title, NULL, 0,
-							  button, GTK_RESPONSE_OK,
-							  NULL));
+	dialog = GTK_DIALOG (eel_alert_dialog_new (NULL,
+		                                   0,
+		                                   GTK_MESSAGE_INFO,
+		                                   GTK_BUTTONS_NONE,
+		                                   wait->wait_message,
+		                                   _("You can stop this operation by clicking cancel.")));
+							
+	gtk_dialog_add_button (GTK_DIALOG (dialog), button, GTK_RESPONSE_OK);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
 	/* The contents are often very small, causing tiny little
 	 * dialogs with their titles clipped if you just let gtk
@@ -224,10 +224,8 @@ timed_wait_callback (gpointer callback_data)
 	gtk_window_set_default_size (GTK_WINDOW (dialog),
 				     TIMED_WAIT_MINIMUM_DIALOG_WIDTH,
 				     -1);
-	gtk_window_set_wmclass (GTK_WINDOW (dialog), "dialog", "Eel");
-	add_label_to_dialog (dialog, wait->wait_message);
 	wait->dialog_creation_time = eel_get_system_time ();
-	gtk_widget_show_all (GTK_WIDGET (dialog));
+	gtk_widget_show (GTK_WIDGET (dialog));
 
 	/* FIXME bugzilla.eazel.com 2441: 
 	 * Could parent here, but it's complicated because we
@@ -242,6 +240,9 @@ timed_wait_callback (gpointer callback_data)
 	g_signal_connect (dialog, "destroy",
 			  G_CALLBACK (timed_wait_dialog_destroy_callback),
 			  wait);
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (trash_dialog_response_callback),
+			  wait);
 
 	wait->timeout_handler_id = 0;
 	wait->dialog = dialog;
@@ -253,20 +254,17 @@ void
 eel_timed_wait_start_with_duration (int duration,
 					 EelCancelCallback cancel_callback,
 					 gpointer callback_data,
-					 const char *window_title,
 					 const char *wait_message,
 					 GtkWindow *parent_window)
 {
 	TimedWait *wait;
 	
 	g_return_if_fail (callback_data != NULL);
-	g_return_if_fail (window_title != NULL);
 	g_return_if_fail (wait_message != NULL);
 	g_return_if_fail (parent_window == NULL || GTK_IS_WINDOW (parent_window));
 
 	/* Create the timed wait record. */
 	wait = g_new0 (TimedWait, 1);
-	wait->window_title = g_strdup (window_title);
 	wait->wait_message = g_strdup (wait_message);
 	wait->cancel_callback = cancel_callback;
 	wait->callback_data = callback_data;
@@ -277,7 +275,7 @@ eel_timed_wait_start_with_duration (int duration,
 	}
 
 	/* Start the timer. */
-	wait->timeout_handler_id = gtk_timeout_add (duration, timed_wait_callback, wait);
+	wait->timeout_handler_id = g_timeout_add (duration, timed_wait_callback, wait);
 
 	/* Put in the hash table so we can find it later. */
 	if (timed_wait_hash_table == NULL) {
@@ -292,14 +290,13 @@ eel_timed_wait_start_with_duration (int duration,
 void
 eel_timed_wait_start (EelCancelCallback cancel_callback,
 			   gpointer callback_data,
-			   const char *window_title,
 			   const char *wait_message,
 			   GtkWindow *parent_window)
 {
 	eel_timed_wait_start_with_duration
 		(TIMED_WAIT_STANDARD_DURATION,
 		 cancel_callback, callback_data,
-		 window_title, wait_message, parent_window);
+		 wait_message, parent_window);
 }
 
 void
@@ -320,19 +317,10 @@ eel_timed_wait_stop (EelCancelCallback cancel_callback,
 	timed_wait_free (wait);
 }
 
-static gboolean
-delete_event_callback (gpointer data,
-		       gpointer user_data)
-{
-	g_return_val_if_fail (GTK_IS_DIALOG (data), FALSE);
-	g_signal_stop_emission_by_name (G_OBJECT (data),
-					"delete_event");
-	return TRUE;
-}
-
 int
 eel_run_simple_dialog (GtkWidget *parent, gboolean ignore_close_box,
-		       const char *text, const char *title, ...)
+		       GtkMessageType message_type, const char *primary_text,
+		       const char *secondary_text, ...)
 {
 	va_list button_title_args;
 	const char *button_title;
@@ -351,9 +339,14 @@ eel_run_simple_dialog (GtkWidget *parent, gboolean ignore_close_box,
 	}
 	
 	/* Create the dialog. */
-        dialog = gtk_dialog_new ();
-	gtk_window_set_title (GTK_WINDOW (dialog), title);
-	va_start (button_title_args, title);
+	dialog = eel_alert_dialog_new (GTK_WINDOW (chosen_parent), 
+	                               0,
+	                               message_type,
+	                               GTK_BUTTONS_NONE,
+	                               primary_text,
+	                               secondary_text);
+	
+	va_start (button_title_args, secondary_text);
 	response_id = 0;
 	while (1) {
 		button_title = va_arg (button_title_args, const char *);
@@ -361,28 +354,15 @@ eel_run_simple_dialog (GtkWidget *parent, gboolean ignore_close_box,
 			break;
 		}
 		gtk_dialog_add_button (GTK_DIALOG (dialog), button_title, response_id);
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog), response_id);
 		response_id++;
 	}
 	va_end (button_title_args);
-	
-	/* Allow close. */
-	if (ignore_close_box) {
-		g_signal_connect (dialog,
-				    "delete_event",
-				    G_CALLBACK (delete_event_callback),
-				    NULL);
-
-	}
-	
-	gtk_window_set_wmclass (GTK_WINDOW (dialog), "dialog", "Eel");
-
-	/* Title it if asked to. */
-	add_label_to_dialog (GTK_DIALOG (dialog), text);
 
 	/* Run it. */
-        gtk_widget_show_all (dialog);
+        gtk_widget_show (dialog);
         result = gtk_dialog_run (GTK_DIALOG (dialog));
-	while (result == -1 && ignore_close_box) {
+	while ((result == GTK_RESPONSE_NONE || result == GTK_RESPONSE_DELETE_EVENT) && ignore_close_box) {
 		gtk_widget_show (GTK_WIDGET (dialog));
 		result = gtk_dialog_run (GTK_DIALOG (dialog));
 	}
@@ -392,190 +372,149 @@ eel_run_simple_dialog (GtkWidget *parent, gboolean ignore_close_box,
 }
 
 static GtkDialog *
-create_message_dialog (const char *message,
-		       const char *dialog_title,
+create_message_dialog (const char *primary_text,
+		       const char *secondary_text,
 		       GtkMessageType type,
 		       GtkButtonsType buttons_type,
 		       GtkWindow *parent)
 {  
-	GtkWidget *box;
+	GtkWidget *dialog;
 
-	g_assert (dialog_title != NULL);
-
-	box = gtk_message_dialog_new (parent, 0, type, buttons_type, "%s", message);
-	gtk_window_set_title (GTK_WINDOW (box), dialog_title);
-	gtk_window_set_wmclass (GTK_WINDOW (box), "stock_dialog", "Eel");
-	
-	return GTK_DIALOG (box);
+	dialog = eel_alert_dialog_new (parent,
+	                               0,
+	                               type,
+	                               buttons_type,
+	                               primary_text,
+	                               secondary_text);
+	return GTK_DIALOG (dialog);
 }
 
 static GtkDialog *
-show_message_dialog (const char *message,
-		     const char *dialog_title,
+show_message_dialog (const char *primary_text,
+		     const char *secondary_text,
 		     GtkMessageType type,
 		     GtkButtonsType buttons_type,
-		     const char *additional_button_label,
-		     int additional_button_response_id,
+		     const char *details_text,
 		     GtkWindow *parent)
 {
 	GtkDialog *dialog;
-	GtkWidget *additional_button;
-	GtkWidget *box;
 
-	dialog = create_message_dialog (message, dialog_title, type, 
+	dialog = create_message_dialog (primary_text, secondary_text, type, 
 					buttons_type, parent);
-	if (additional_button_label != NULL) {
-		additional_button = gtk_dialog_add_button (dialog,
-							   additional_button_label,
-							   additional_button_response_id);
-		box = gtk_widget_get_parent (additional_button);
-		if (GTK_IS_BOX (box)) {
-			gtk_box_reorder_child (GTK_BOX (box), additional_button, 0);
-		}
+	if (details_text != NULL) {
+		eel_alert_dialog_set_details_label (EEL_ALERT_DIALOG (dialog), details_text);
 	}
 	gtk_widget_show (GTK_WIDGET (dialog));
 
-	return dialog;
-}
-
-static GtkDialog *
-show_ok_dialog (const char *message,
-		const char *dialog_title,
-		GtkMessageType type,
-		GtkWindow *parent)
-{  
-	GtkDialog *dialog;
-
-	dialog = show_message_dialog (message, dialog_title, type,
-				      GTK_BUTTONS_OK, NULL, 0, parent);
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
-	
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (gtk_object_destroy), NULL);
 
 	return dialog;
 }
 
+static GtkDialog *
+show_ok_dialog (const char *primary_text,
+		const char *secondary_text,
+		GtkMessageType type,
+		GtkWindow *parent)
+{  
+	GtkDialog *dialog;
+
+	dialog = show_message_dialog (primary_text, secondary_text, type,
+				      GTK_BUTTONS_OK, NULL, parent);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+	
+	return dialog;
+}
+
 GtkDialog *
-eel_create_info_dialog (const char *info,
-			const char *dialog_title,
+eel_create_info_dialog (const char *primary_text,
+			const char *secondary_text,
 			GtkWindow *parent)
 {
-	return create_message_dialog (info, dialog_title, 
+	return create_message_dialog (primary_text, secondary_text,
 				      GTK_MESSAGE_INFO,
 				      GTK_BUTTONS_OK,
 				      parent);
 }
 
 GtkDialog *
-eel_show_info_dialog (const char *info,
-		      const char *dialog_title,
+eel_show_info_dialog (const char *primary_text,
+		      const char *secondary_text,
 		      GtkWindow *parent)
 {
-	return show_ok_dialog (info, 
-			    dialog_title == NULL ? _("Info") : dialog_title, 
+	return show_ok_dialog (primary_text, 
+			    secondary_text,
 			    GTK_MESSAGE_INFO, parent);
 }
 
-static void
-details_dialog_response_callback (GtkDialog *dialog,
-				  int response_id,
-				  const char *detailed_message)
-{
-	if (response_id == RESPONSE_DETAILS) {
-		gtk_label_set_text (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label),
-				    detailed_message);
-		gtk_dialog_set_response_sensitive (dialog, RESPONSE_DETAILS, FALSE);
-	} else {
-		gtk_object_destroy (GTK_OBJECT (dialog));
-	}
-}
-
-
 GtkDialog *
-eel_show_info_dialog_with_details (const char *info,
-				   const char *dialog_title,
+eel_show_info_dialog_with_details (const char *primary_text,
+				   const char *secondary_text,
 				   const char *detailed_info,
 				   GtkWindow *parent)
 {
 	GtkDialog *dialog;
 
 	if (detailed_info == NULL
-	    || strcmp (info, detailed_info) == 0) {
-		return eel_show_info_dialog (info, dialog_title, parent);
+	    || strcmp (primary_text, detailed_info) == 0) {
+		return eel_show_info_dialog (primary_text, secondary_text, parent);
 	}
 
-	dialog = show_message_dialog (info,
-				      dialog_title == NULL ? _("Info") : dialog_title, 
+	dialog = show_message_dialog (primary_text,
+				      secondary_text,
 				      GTK_MESSAGE_INFO, 
-				      GTK_BUTTONS_OK, _("Details"), RESPONSE_DETAILS,
+				      GTK_BUTTONS_OK,
+				      detailed_info,
 				      parent);
 
-
-	g_signal_connect_closure (G_OBJECT (dialog),
-				  "response",
-				  g_cclosure_new (
-					  G_CALLBACK (details_dialog_response_callback),
-					  g_strdup (detailed_info),  
-					  (GClosureNotify) g_free),
-				  FALSE);
-
 	return dialog;
 
 }
 
 
 GtkDialog *
-eel_show_warning_dialog (const char *warning,
-			 const char *dialog_title,
+eel_show_warning_dialog (const char *primary_text,
+			 const char *secondary_text,
 			 GtkWindow *parent)
 {
-	return show_ok_dialog (warning, 
-			       dialog_title == NULL ? _("Warning") : dialog_title, 
+	return show_ok_dialog (primary_text, 
+			       secondary_text,
 			       GTK_MESSAGE_WARNING, parent);
 }
 
 
 GtkDialog *
-eel_show_error_dialog (const char *error,
-		       const char *dialog_title,
+eel_show_error_dialog (const char *primary_text,
+		       const char *secondary_text,
 		       GtkWindow *parent)
 {
-	return show_ok_dialog (error,
-			       dialog_title == NULL ? _("Error") : dialog_title, 
+	return show_ok_dialog (primary_text,
+			       secondary_text,
 			       GTK_MESSAGE_ERROR, parent);
 }
 
-
 GtkDialog *
-eel_show_error_dialog_with_details (const char *error_message,
-				    const char *dialog_title,
+eel_show_error_dialog_with_details (const char *primary_text,
+				    const char *secondary_text,
 				    const char *detailed_error_message,
 				    GtkWindow *parent)
 {
 	GtkDialog *dialog;
 
-	g_return_val_if_fail (error_message != NULL, NULL);
+	g_return_val_if_fail (primary_text != NULL, NULL);
 	g_return_val_if_fail (parent == NULL || GTK_IS_WINDOW (parent), NULL);
 
 	if (detailed_error_message == NULL
-	    || strcmp (error_message, detailed_error_message) == 0) {
-		return eel_show_error_dialog (error_message, dialog_title, parent);
+	    || strcmp (primary_text, detailed_error_message) == 0) {
+		return eel_show_error_dialog (primary_text, secondary_text, parent);
 	}
 	
-	dialog = show_message_dialog (error_message, 
-				      dialog_title == NULL ? _("Error") : dialog_title,
+	dialog = show_message_dialog (primary_text, 
+				      secondary_text,
 				      GTK_MESSAGE_ERROR,
-				      GTK_BUTTONS_OK, _("Details"), RESPONSE_DETAILS,
+				      GTK_BUTTONS_OK, detailed_error_message,
 				      parent);
-	
-	/* Show the details when you click on the details button. */
-	g_signal_connect_closure (G_OBJECT (dialog), "response",
-				  g_cclosure_new (
-					  G_CALLBACK (details_dialog_response_callback),
-					  g_strdup (detailed_error_message),  
-					  (GClosureNotify) g_free),
-				  FALSE);
 	return dialog;
 }
 
@@ -592,15 +531,15 @@ eel_show_error_dialog_with_details (const char *error_message,
  * @parent: The parent window for this dialog.
  */
 GtkDialog *
-eel_show_yes_no_dialog (const char *question, 
-			const char *dialog_title,
+eel_show_yes_no_dialog (const char *primary_text, 
+			const char *secondary_text,
 			const char *yes_label,
 			const char *no_label,
 			GtkWindow *parent)
 {
-	GtkDialog *dialog = 0;
-	dialog = eel_create_question_dialog (question,
-					     dialog_title == NULL ? _("Question") : dialog_title,
+	GtkDialog *dialog = NULL;
+	dialog = eel_create_question_dialog (primary_text,
+					     secondary_text,
 					     no_label, GTK_RESPONSE_CANCEL,
 					     yes_label, GTK_RESPONSE_YES,
 					     GTK_WINDOW (parent));
@@ -622,8 +561,8 @@ eel_show_yes_no_dialog (const char *question,
  * @parent: The parent window for this dialog.
  */
 GtkDialog *
-eel_create_question_dialog (const char *question,
-			    const char *dialog_title,
+eel_create_question_dialog (const char *primary_text,
+			    const char *secondary_text,
 			    const char *answer_1,
 			    int response_1,
 			    const char *answer_2,
@@ -632,11 +571,12 @@ eel_create_question_dialog (const char *question,
 {
 	GtkDialog *dialog;
 	
-	dialog = create_message_dialog (question,
-					dialog_title == NULL ? _("Question") : dialog_title, 
+	dialog = create_message_dialog (primary_text,
+					secondary_text,
 					GTK_MESSAGE_QUESTION,
 					GTK_BUTTONS_NONE,
 					parent);
 	gtk_dialog_add_buttons (dialog, answer_1, response_1, answer_2, response_2, NULL);
+	gtk_dialog_set_default_response (dialog, response_2);
 	return dialog;
 }
